@@ -14,7 +14,7 @@
 
 ;;; Commentary:
 
-;; This package is a minor mode for editing iPerl version 0.6 documents.
+;; This package is a minor mode for editing iPerl documents.
 
 ;;; Code:
 
@@ -33,7 +33,8 @@
 
 
 (defcustom iPerl-text-face nil
-  "*Face that highlights plain text parts of an iPerl document or nil."
+  "*Face that highlights text parts of an iPerl document or nil.
+There is no point in setting this if you use font-locking."
   :type 'face
   :group 'iPerl)
 (setplist 'iPerl-text
@@ -104,7 +105,6 @@
 
 ;;; keymaps:
 
-;;;###autoload
 (defcustom iPerl-map-prefix "\C-c!"
   "*Prefix key to use for iPerl commands in iPerl minor mode.
 You might set this to something like [f12] for a function key.
@@ -122,20 +122,19 @@ After that, changing the prefix key requires manipulating keymaps."
 (if iPerl-mode-map
     nil
   (setq iPerl-mode-map (make-sparse-keymap))
-  (define-key iPerl-mode-map "\C-\M-x" 'iPerl-on-region)
 
   (let ((map (make-sparse-keymap)))
     (define-key iPerl-mode-map iPerl-map-prefix map)
     (mapcar
      (lambda (item) (define-key map (car item) (cdr item)))
-     '(([right] . iPerl-forward)
+     '(("@" . iPerl-mark-subtree)
+       ([right] . iPerl-forward)
        ("\C-f" . iPerl-forward)
        ([left] . iPerl-backward)
        ("\C-b" . iPerl-backward)
        ("\C-p" . iPerl-hide-perl)
        ("\C-\M-p" . iPerl-hide-perl-and-markup)
        ("\M-p" . iPerl-hide-markup)
-       ("\M-m" . iPerl-hide-markup)
        ("\C-t" . iPerl-hide-text)
        ("\C-s" . iPerl-show-all)
        ("b" . iPerl-bit)
@@ -158,11 +157,7 @@ After that, changing the prefix key requires manipulating keymaps."
 		("Bit of Perl" . iPerl-bit)
 		("Long Bit of Perl" . iPerl-long-bit)
 		("Line of Perl" . iPerl-line)
-		("Printing Bit of Perl" . iPerl-printing-bit)
-		("--" . 1)
-		("iperl on Region" . iPerl-on-region)
-		("Set Style" . iPerl-set-style)
-		("Toggle iPerl Mode" . iPerl-mode))))))
+		("Printing Bit of Perl" . iPerl-printing-bit))))))
 
 (or (assq 'iPerl-mode minor-mode-map-alist)
     (setq minor-mode-map-alist
@@ -183,12 +178,9 @@ After that, changing the prefix key requires manipulating keymaps."
 When given in a file's local variables section must be a literal string,
 since that is what the iPerl interpreter understands, but as a lisp-variable
 this gets transformed to a symbol as soon as it is used.")
-(put 'iPerl-style 'permanent-local t)
-
 
 (defcustom iPerl-style-equiv
-  '((xml . xml-script)
-    (sgml . xml-script)
+  '((sgml . sgml-script)
     (unix . bang))
   "*Alternate style to search for when car is not found in some alist."
   :type 'list
@@ -211,8 +203,8 @@ this gets transformed to a symbol as soon as it is used.")
 		"P<"		'">"
 		"^=begin perl"	"^=end perl"
 		"^=for perl"	"\n\n\\|\\'")
-    (xml	"<perl>"	"</perl>"
-		"<script[^>]*\\s +runat\\s *=\\s *server[^>]*>" "</script>"
+    (sgml	"<perl>"	"</perl>"
+		"<script[^>]*\\s +runat\\s +=\\s +server[^>]*>" "</script>"
 		"<server>"	"</server>"
 		"<{"		"}>"
 		"<}>"		1
@@ -239,12 +231,6 @@ end-regexp is quoted, the pair surrounds a printing bit of Perl."
   :group 'iPerl)
 
 
-(defvar iPerl-header-marker nil
-  "When non-`nil' is the end of header for prepending by \\[iPerl-on-region].
-That command is also used for setting this variable.")
-
-
-(defvar iPerl-skeleton-filter)
 
 
 
@@ -263,8 +249,8 @@ And then for the rest there is the mode usually used by that document.  These
 parts, as well as the markup between them can be hidden, so as to allow
 concentrating on only one aspect or the other.  They are also graphically set
 apart with the faces `iPerl-text-face', `iPerl-markup-open-face',
-`iPerl-perl-face', `iPerl-printing-perl-face' and `iPerl-markup-close-face' to
-be distinguishable at a glance.
+`iPerl-perl-face' and `iPerl-markup-close-face' to be distinguishable at a
+glance.
 
 All standard styles are supported, and the markup between text and Perl can be
 automatically inserted in the syntax of the current style.  When you add
@@ -293,53 +279,36 @@ Info on iPerl and latest version are at http://beam.to/iPerl/"
 		  nil t)
 	(make-local-variable 'line-move-ignore-invisible)
 	(make-local-variable 'skeleton-further-elements)
+	(make-local-variable 'skeleton-filter)
 	(make-local-variable 'iPerl-style)
-	(make-local-variable 'iPerl-header-marker)
 	(setq line-move-ignore-invisible t
 	      skeleton-further-elements
-	        '((space '(if iPerl-skeleton-space ?\ ))))
-	;; (buffer-local-variables &optional BUFFER)
-	(if (local-variable-p 'skeleton-filter)
-	    (progn
-	      (make-local-variable 'iPerl-skeleton-filter)
-	      (setq iPerl-skeleton-filter skeleton-filter
-		    skeleton-filter
-		      (lambda (alist)
-			(or (iPerl-by-style alist t)
-			    (funcall iPerl-skeleton-filter alist)))))
-	  (make-local-variable 'skeleton-filter)
-	  (setq skeleton-filter
-		  (lambda (alist)
-		    (or (iPerl-by-style alist t)
-			alist))))
-	(if iPerl-style
-	    (iPerl-set-style iPerl-style)
-	  (call-interactively 'iPerl-set-style))
+	        '((space '(if iPerl-skeleton-space ?\ )))
+	      skeleton-filter 'iPerl-by-style)
+	(or iPerl-style (call-interactively 'iPerl-set-style))
 	(if (eq buffer-invisibility-spec t)
 	    (setq buffer-invisibility-spec))
+	(iPerl-add-overlays)
 	(run-hooks 'iPerl-minor-mode-hook))
     (save-restriction
       (widen)
       (and ;; (= emacs-major-version 20)
 	   ;; (< emacs-minor-version 7)
-	   (let ((modified (buffer-modified-p)))
+	   (progn
 	     (put-text-property (point-min) (point-max) 'point-entered nil)
-	     (put-text-property (point-min) (point-max) 'point-left nil)
-	     (or modified
-		 (set-buffer-modified-p nil))))
+	     (put-text-property (point-min) (point-max) 'point-left nil)))
       (mapcar
        (lambda (o)
 	 (if (memq (overlay-get o 'category) iPerl-categories)
 	     (delete-overlay o)))
        (overlays-in (point-min) (point-max))))
     (setq line-move-ignore-invisible nil)
-    (remove-from-invisibility-spec iPerl-categories)
-    (force-mode-line-update)))
+    (remove-from-invisibility-spec iPerl-categories))
+  (force-mode-line-update))
 
-;;;###autoload
 (defun iPerl-mode-if-style ()
   "Turn on iPerl-mode if `iPerl-style' is set.
-This can be added to `hack-local-variables-hook' for files that have this in
+This can be added to `change-major-mode-hook' for files that have this in
 their local variables section."
   (and (boundp 'iPerl-style)
        iPerl-style
@@ -351,16 +320,10 @@ their local variables section."
   (interactive
    (let ((style (completing-read "Style: (default bang) "
 				 '(("bang") ("control") ("cpp") ("m4")
-				   ("pod") ("sgml") ("xml") ("unix"))
+				   ("pod") ("sgml") ("unix"))
 				 nil t)))
      (if (string= style "") '("bang") (list style))))
-  (mapcar
-   (lambda (o)
-     (if (memq (overlay-get o 'category) iPerl-categories)
-	 (delete-overlay o)))
-   (overlays-in (point-min) (point-max)))
-  (setq iPerl-style style)
-  (iPerl-add-overlays))
+  (setq iPerl-style style))
 
 (defun iPerl-by-style (alist &optional noerr)
   ;; choose from ALIST by iPerl-style, possibly helped by iPerl-style-equiv
@@ -369,9 +332,8 @@ their local variables section."
   (or (assq iPerl-style alist)
       (if (assq iPerl-style iPerl-style-equiv)
 	  (assq (cdr (assq iPerl-style iPerl-style-equiv)) alist))
-      (if noerr
-	  ()
-	(error "Not defined for style \"%s\"." iPerl-style))))
+      noerr
+      (error "Not defined for style \"%s\"." iPerl-style)))
 
 
 
@@ -438,10 +400,8 @@ their local variables section."
 			     'iPerl-markup-close))
 	    (setq p (match-end 0)))))
       (iPerl-overlay p (point-max) 'iPerl-text t)
-      (or modified
-	  (set-buffer-modified-p nil)))
-    (run-hooks 'iPerl-view-change-hook))
-  (force-mode-line-update))
+      (set-buffer-modified-p modified))
+    (run-hooks 'iPerl-view-change-hook)))
 
 
 
@@ -547,35 +507,6 @@ See `iPerl-intangible-when-hidden'."
 
 
 
-(defun iPerl-on-region (start end &optional flag)
-  "Pass optional header and region to iperl interpreter.
-The header feature is for adding defines, includes and such to the region.
-With a positive prefix ARG, instead of sending region, define header from
-beginning of buffer to point.  With a negative prefix ARG, instead of sending
-region, clear header."
-  (interactive "r\nP")
-  (if flag
-      (setq iPerl-header-marker (if (> (prefix-numeric-value flag) 0)
-				    (point-marker)))
-    (if (and iPerl-header-marker (< iPerl-header-marker start))
-	(save-excursion
-	  (let (buffer-undo-list
-		(modified (buffer-modified-p)))
-	    (goto-char iPerl-header-marker)
-	    (append-to-buffer (current-buffer) start end)
-	    (shell-command-on-region (point-min)
-				     (setq end (+ iPerl-header-marker
-						  (- end start)))
-				     (format "iperl --%s" iPerl-style))
-	    (delete-region iPerl-header-marker end)
-	    (or modified
-		(set-buffer-modified-p nil))))
-      (if iPerl-header-marker
-	  (setq start (point-min)))
-      (shell-command-on-region start end (format "iperl --%s" iPerl-style)))))
-
-
-
 ;; Function to be set as an iPerl-isearch-open-invisible' property
 ;; to the overlay that makes the iPerl invisible.
 (defun iPerl-isearch-open-invisible (overlay)
@@ -589,31 +520,31 @@ region, clear header."
 
 (define-skeleton iPerl-bit
   "Insert markup for a bit of Perl.
-There are four variations of xml, `xml-perl', `xml-script', `xml-server'
-and `xml-short' one of which gets chosen via `iPerl-style-equiv'.
+There are four variations of sgml, `sgml-perl', `sgml-script', `sgml-server'
+and `sgml-short' one of which gets chosen via `iPerl-style-equiv'.
 See `iPerl-skeleton-space'."
   (bang () "!{" space _ space "}!")
   (control () ? space _  space ?)
   (m4 () "perl({" space _ space "})")
   (pod () "P<{" space _ space "}>")
-  (xml-perl () "<perl>" space _ space "</perl>")
-  (xml-script () "<script runat=server>" space _ space "</script>")
-  (xml-server () "<server>" space _ space "</server>")
-  (xml-short () "<{" space _ space "}>"))
+  (sgml-perl () "<perl>" space _ space "</perl>")
+  (sgml-script () "<script runat=server>" space _ space "</script>")
+  (sgml-server () "<server>" space _ space "</server>")
+  (sgml-short () "<{" space _ space "}>"))
 
 (define-skeleton iPerl-long-bit
   "Insert markup for a multiline bit of Perl.
-See `iPerl-bit' for xml possiblities and `iPerl-skeleton-space'."
+See `iPerl-bit' for sgml possiblities and `iPerl-skeleton-space'."
   (bang () "!{\n" _ "\n}!")
   (control () "\n" _ "\n")
   (m4 () "perl({\n" _ "\n})")
   (pod () '(if (bolp) ?\n "\n\n")
        "=begin perl\n" _ "\n\n=end perl"
        '(if (eolp) ?\n "\n\n"))
-  (xml-perl () "<perl>\n" _ "\n</perl>")
-  (xml-script () "<script runat=server>\n" _ "\n</script>")
-  (xml-server () "<server>\n" _ "\n</server>")
-  (xml-short () "<{\n" _ "\n}>"))
+  (sgml-perl () "<perl>\n" _ "\n</perl>")
+  (sgml-script () "<script runat=server>\n" _ "\n</script>")
+  (sgml-server () "<server>\n" _ "\n</server>")
+  (sgml-short () "<{\n" _ "\n}>"))
 
 (define-skeleton iPerl-printing-bit
   "Insert markup for a printing bit of Perl.
@@ -622,7 +553,7 @@ See `iPerl-skeleton-space'."
   (control () ? space _  space ?)
   (m4 () "perl(<" space _ space ">)")
   (pod () "P<" space _ space ?>)
-  (xml () "&<" space _ space ">;"))
+  (sgml () "&<" space _ space ">;"))
 
 (define-skeleton iPerl-line
   "Insert markup for a line of Perl.
@@ -632,65 +563,4 @@ See `iPerl-skeleton-space'."
   (cpp () ?# space)
   (pod () "=for perl" space))
 
-(provide 'iPerl)
-
-
-
-[					; lisp half ignores POD this way
-
-=head1 NAME
-
-iPerl.el - Edit iPerl documents
-with comfortable support in Emacs
-
-
-=head1 DESCRIPTION
-
-This library allows editing iPerl documents in two modes simultaneously.
-There is the major mode for the host document.  And then there is iPerl minor
-mode, which will enhance the current major mode.  The bits of markup and
-Perl in your document are edited in Perl or CPerl mode.  The menus and
-keybindings change automatically when the cursor is on a bit of Perl.
-
-And, so you see what you are doing, markup, plain bits and printing bits of
-Perl are each highlighted in a different color, and you can make markup and/or
-Perl or the rest of the text invisible so as to concentrate on one aspect of
-your document or the other.
-
-The documentation for this is all in the Emacs help system.  As a good entry
-point (if the minor mode is active) click Describe Buffer Modes from the Help
-menu.  Or use Describe Function... and type in C<iPerl-mode>.
-
-
-
-=head1 INSTALLATION
-
-Click Describe Variable... from the Help menu and type in C<load-path>.  You
-get a list of directories where Emacs looks for libraries.  You should copy
-F<lisp/iPerl.elc> and optionally the source file F<lisp/iPerl.el> to one of
-them.  Emacs foresees the directory containing F<site-lisp> for this.
-
-In your file F<~/.emacs> add the following:
-
-  (setq iPerl-map-prefix "\C-c!")
-  (setq iPerl-map-prefix [f12])
-
-  (let ((map (make-sparse-keymap)))
-    (define-key map "\C-m" 'iPerl-mode)
-    (global-set-key iPerl-map-prefix map))
-
-  (autoload 'iPerl-mode "iPerl" "Toggle iPerl minor mode." t nil)
-
-Of the first two lines you can chose one or the other.  The first one means,
-along the guidelines for minor-mode keybindings, that the key sequence C<C-c !
-C-m> (that's C<control-C ! control-M>) will activate and deactivate iPerl
-minor mode.  The second more conveniently means that C<f12 C-m> (that's C<F12
-control-M>) will do it.  This is then also the keybinding prefix for all
-commands of iPerl minor mode.
-
-
-=head1 SEE ALSO
-
-L<Text::iPerl>, L<iperl>, L<web-iPerl>, L<emacs>, L<perl>, http://beam.to/iPerl/
-
-=cut ]
\ No newline at end of file
+(provide 'iPerl)
\ No newline at end of file
